-- CS422 Summer 2004
-- Final
-- C.S.

-- [Schema Creation] --

create table students (
	sid	int primary key,
	name	varchar(64),
	dropped	boolean
);

insert into students values (1000000, 'Peter Paker', 'f');
insert into students values (1000001, 'Mary Jane Watson', 'f');

create table assignments (
	lname	varchar(128) unique,
	sname	varchar(16) primary key,
	total	int,
	due	timestamp
);

insert into assignments values ('Project 0', 'proj0', 80, '2004-8-1 11:59pm');
insert into assignments values ('Project 1', 'proj1', 120, '2004-9-9 11:59pm');

create table files (
	name		varchar(128),
	size		int,
	sid		int references students(sid),
	assignment	varchar(16) references assignments(sname),
  primary key (sid, assignment, name)
);

create table grades (
	sid		int references students(sid),
	assignment	varchar(16) references assignments(sname),
	points		int,
  primary key (sid, assignment)
);

-- [Triggers] --

-- (a) anti_dos

create or replace function anti_dos() returns trigger as '
declare
	current_total_size int;
begin
	select into current_total_size sum(size) as ctz from files
		where sid = new.sid and assignment = new.assignment;
	if current_total_size + new.size > 2 * 1024 * 1024 then
		RAISE EXCEPTION ''file limit exceeded'';
	end if;
	return new;
end;
' language plpgsql;

create trigger anti_dos_trig
	before insert on files
	for each row
	execute procedure anti_dos();

insert into files values ( 'file1', 1024102, 1000000, 'proj1' );
insert into files values ( 'file2', 1024102, 1000000, 'proj1' );
insert into files values ( 'file3', 1024102, 1000000, 'proj1' );
insert into files values ( 'file1', 1024102, 1000001, 'proj1' );

-- (b) no_late_turnin

create or replace function no_late_turnin() returns trigger as '
declare
	due_time timestamp;
begin
	select into due_time due from assignments where sname = new.assignment;
	if due_time < current_timestamp then
		RAISE EXCEPTION ''past due'';
	end if;
	return new;
end;
' language plpgsql;

create trigger no_late_turnin_trig
	before insert on files
	for each row
	execute procedure no_late_turnin();

insert into files values ( 'file4', 1024, 1000000, 'proj0' );
insert into files values ( 'file4', 1024, 1000001, 'proj0' );
insert into files values ( 'file4', 1024, 1000001, 'proj1' );

-- [Functions] --

-- (a) grade_percentage

create or replace function grade_percentage (int) returns float as '
declare
	midterm_total	float;
	midterm_points	float;
	final_total	float;
	final_points	float;
	other_total	float;
	other_points	float;
begin
	-- midterms
	select into midterm_total coalesce(sum(total),1) as mt from assignments
		where sname like ''midterm%'';
	select into midterm_points coalesce(sum(points),0) as mp from grades
		where sid = $1 and assignment like ''midterm%'';

	-- final
	select into final_total coalesce(sum(total),1) as ft from assignments
		where sname = ''final'';
	select into final_points coalesce(sum(points),0) as fp from grades
		where sid = $1 and assignment = ''final'';

	-- other assignments
	select into other_total coalesce(sum(total),1) as ot from assignments
		where sname not like ''midter%'' and sname <> ''final'';
	select into other_points coalesce(sum(points),0) as op from grades
		where sid = $1 and assignment not like ''midterm%''
		and assignment <> ''final'';

	return	0.3 * midterm_points / midterm_total +
		0.3 * final_points / final_total +
		0.4 * other_points / other_total;
end;
' language plpgsql;

insert into assignments values ('Midterm 1', 'midterm1', 100, '2004-09-01');
insert into assignments values ('Midterm 2', 'midterm2', 100, '2004-09-01');
insert into assignments values ('Final', 'final', 100, '2004-09-01');
insert into grades values (1000000, 'midterm1', 50);
insert into grades values (1000000, 'midterm2', 100);
insert into grades values (1000000, 'final', 85);

-- (b) course_grade

create or replace function course_grade (int) returns varchar as '
declare
	gp	float;
begin
	gp := grade_percentage( $1 );
	if gp >= 0.85 then
		return ''A'';
	elsif gp >= 0.7 then
		return ''B'';
	elsif gp >= 0.6 then
		return ''C'';
	elsif gp >= 0.45 then
		return ''D'';
	else
		return ''F'';
	end if;
end;
' language plpgsql;

-- [Queries] --

select sid, name, course_grade(sid) as grade from students order by grade asc;

select max(points), min(points), avg(points) from grades
	where assignment = 'hw4';

select sid from grades where abs(points-83) =
	(select min(abs(points-83)) from grades where assignment = 'midterm1');

select assignment, count(distinct sid) from files
	group by assignment
	having count(distinct sid) < 2 / 3 *
		(select count(*) from students where dropped <> 't')
union
select a.sname, 0 from assignments a left join files f
	on a.sname = f.assignment
	where f.sid is null;

